import 'data_client.dart';
import 'model.dart';

abstract class IRestService<T> {
  Future<T> getById(int id);

  Future<List<T>> getAll();

  Future<List<T>> filter({Map<String, dynamic> queryParameters});

  Future<void> insert(T model);

  Future<void> update(T model, int id);

  Future<void> delete(int id);

  Future<void> patch(T model, int id);

  T fromMap(dynamic map);
}

abstract class RestService<T extends Model> implements IRestService<T> {
  IDataClient dataClient;

  String get path;

  RestService(IDataClient dataClient) {
    this.dataClient = dataClient;
  }

  Future<T> getById(int id) async {
    var data = await dataClient.get(
      "$path/$id",
    );

    return fromMap(data);
  }

  Future<List<T>> getAll() async {
    return await _get();
  }

  Future<List<T>> _get({Map<String, dynamic> queryParameters}) async {
    dynamic data = await dataClient.get(path, queryParameters: queryParameters);
    List<T> models = (data as List).map((data) => fromMap(data)).toList();
    return models;
  }

  Future<List<T>> filter({Map<String, dynamic> queryParameters}) async {
    return _get(queryParameters: queryParameters);
  }

  Future<void> insert(T model) async {
    await dataClient.post(path, data: model.toJson());
  }

  Future<void> update(T model, int id) async {
    await dataClient.put("$path/$id", data: model.toJson());
  }

  Future<void> delete(int id) async {
    await dataClient.delete(
      "$path/$id",
    );
  }

  Future<void> patch(T model, int id) async {
    await dataClient.patch("$path/$id", data: model.toJson());
  }
}
